@@ -17,7 +17,6 @@ from letsencrypt import account
 from letsencrypt import auth_handler
 from letsencrypt import configuration
 from letsencrypt import constants
-from letsencrypt import continuity_auth
 from letsencrypt import crypto_util
 from letsencrypt import errors
 from letsencrypt import error_handler
@@ -28,6 +27,7 @@ from letsencrypt import storage
 
 from letsencrypt.display import ops as display_ops
 from letsencrypt.display import enhancements
+from letsencrypt.plugins import selection as plugin_selection
 
 
 logger = logging.getLogger(__name__)
@@ -105,7 +105,8 @@ def register(config, account_storage, tos_cb=None):
                    "--register-unsafely-without-email was not present.")
             logger.warn(msg)
             raise errors.Error(msg)
-        logger.warn("Registering without email!")
+        if not config.dry_run:
+            logger.warn("Registering without email!")
 
     # Each new registration shall use a fresh new key
     key = jose.JWKRSA(key=jose.ComparableRSAKey(
@@ -146,9 +147,8 @@ def perform_registration(acme, config):
     """
     try:
         return acme.register(messages.NewRegistration.from_data(email=config.email))
-    except messages.Error, e:
-        err = repr(e)
-        if "MX record" in err or "Validation of contact mailto" in err:
+    except messages.Error as e:
+        if e.typ == "urn:acme:error:invalidEmail":
             config.namespace.email = display_ops.get_email(more=True, invalid=True)
             return perform_registration(acme, config)
         else:
@@ -161,21 +161,21 @@ class Client(object):
     :ivar .IConfig config: Client configuration.
     :ivar .Account account: Account registered with `register`.
     :ivar .AuthHandler auth_handler: Authorizations handler that will
-        dispatch DV and Continuity challenges to appropriate
-        authenticators (providing `.IAuthenticator` interface).
-    :ivar .IAuthenticator dv_auth: Prepared (`.IAuthenticator.prepare`)
-        authenticator that can solve the `.constants.DV_CHALLENGES`.
+        dispatch DV challenges to appropriate authenticators
+        (providing `.IAuthenticator` interface).
+    :ivar .IAuthenticator auth: Prepared (`.IAuthenticator.prepare`)
+        authenticator that can solve ACME challenges.
     :ivar .IInstaller installer: Installer.
     :ivar acme.client.Client acme: Optional ACME client API handle.
        You might already have one from `register`.
 
     """
 
-    def __init__(self, config, account_, dv_auth, installer, acme=None):
+    def __init__(self, config, account_, auth, installer, acme=None):
         """Initialize a client."""
         self.config = config
         self.account = account_
-        self.dv_auth = dv_auth
+        self.auth = auth
         self.installer = installer
 
         # Initialize ACME if account is provided
@@ -183,19 +183,14 @@ class Client(object):
             acme = acme_from_config_key(config, self.account.key)
         self.acme = acme
 
-        # TODO: Check if self.config.enroll_autorenew is None. If
-        # so, set it based to the default: figure out if dv_auth is
-        # standalone (then default is False, otherwise default is True)
-
-        if dv_auth is not None:
-            cont_auth = continuity_auth.ContinuityAuthenticator(config,
-                                                                installer)
+        if auth is not None:
             self.auth_handler = auth_handler.AuthHandler(
-                dv_auth, cont_auth, self.acme, self.account)
+                auth, self.acme, self.account)
         else:
             self.auth_handler = None
 
-    def _obtain_certificate(self, domains, csr):
+    def obtain_certificate_from_csr(self, domains, csr,
+        typ=OpenSSL.crypto.FILETYPE_ASN1, authzr=None):
         """Obtain certificate.
 
         Internal function with precondition that `domains` are
@@ -205,6 +200,8 @@ class Client(object):
         :param .le_util.CSR csr: DER-encoded Certificate Signing
             Request. The key used to generate this CSR can be different
             than `authkey`.
+        :param list authzr: List of
+            :class:`acme.messages.AuthorizationResource`
 
         :returns: `.CertificateResource` and certificate chain (as
             returned by `.fetch_chain`).
@@ -221,35 +218,21 @@ class Client(object):
 
         logger.debug("CSR: %s, domains: %s", csr, domains)
 
-        authzr = self.auth_handler.get_authorizations(domains)
+        if authzr is None:
+            authzr = self.auth_handler.get_authorizations(domains)
+
         certr = self.acme.request_issuance(
-            jose.ComparableX509(OpenSSL.crypto.load_certificate_request(
-                OpenSSL.crypto.FILETYPE_ASN1, csr.data)),
-            authzr)
+            jose.ComparableX509(
+                OpenSSL.crypto.load_certificate_request(typ, csr.data)),
+                authzr)
         return certr, self.acme.fetch_chain(certr)
 
-    def obtain_certificate_from_csr(self, csr):
-        """Obtain certficiate from CSR.
-
-        :param .le_util.CSR csr: DER-encoded Certificate Signing
-            Request.
-
-        :returns: `.CertificateResource` and certificate chain (as
-            returned by `.fetch_chain`).
-        :rtype: tuple
-
-        """
-        return self._obtain_certificate(
-            # TODO: add CN to domains?
-            crypto_util.get_sans_from_csr(
-                csr.data, OpenSSL.crypto.FILETYPE_ASN1), csr)
-
     def obtain_certificate(self, domains):
         """Obtains a certificate from the ACME server.
 
         `.register` must be called before `.obtain_certificate`
 
-        :param set domains: domains to get a certificate
+        :param list domains: domains to get a certificate
 
         :returns: `.CertificateResource`, certificate chain (as
             returned by `.fetch_chain`), and newly generated private key
@@ -258,12 +241,20 @@ class Client(object):
         :rtype: tuple
 
         """
+        authzr = self.auth_handler.get_authorizations(
+                domains,
+                self.config.allow_subset_of_names)
+
+        domains = [a.body.identifier.value.encode('ascii')
+                                          for a in authzr]
+
         # Create CSR from names
         key = crypto_util.init_save_key(
             self.config.rsa_key_size, self.config.key_dir)
         csr = crypto_util.init_save_csr(key, domains, self.config.csr_dir)
 
-        return self._obtain_certificate(domains, csr) + (key, csr)
+        return (self.obtain_certificate_from_csr(domains, csr, authzr=authzr)
+                                                                + (key, csr))
 
     def obtain_and_enroll_certificate(self, domains):
         """Obtain and enroll certificate.
@@ -276,34 +267,28 @@ class Client(object):
         :param plugins: A PluginsFactory object.
 
         :returns: A new :class:`letsencrypt.storage.RenewableCert` instance
-            referred to the enrolled cert lineage, or False if the cert could
-            not be obtained.
+            referred to the enrolled cert lineage, False if the cert could not
+            be obtained, or None if doing a successful dry run.
 
         """
         certr, chain, key, _ = self.obtain_certificate(domains)
 
-        # XXX: We clearly need a more general and correct way of getting
-        # options into the configobj for the RenewableCert instance.
-        # This is a quick-and-dirty way to do it to allow integration
-        # testing to start.  (Note that the config parameter to new_lineage
-        # ideally should be a ConfigObj, but in this case a dict will be
-        # accepted in practice.)
-        params = vars(self.config.namespace)
-        config = {}
-        cli_config = configuration.RenewerConfiguration(self.config.namespace)
-
-        if (cli_config.config_dir != constants.CLI_DEFAULTS["config_dir"] or
-                cli_config.work_dir != constants.CLI_DEFAULTS["work_dir"]):
+        if (self.config.config_dir != constants.CLI_DEFAULTS["config_dir"] or
+                self.config.work_dir != constants.CLI_DEFAULTS["work_dir"]):
             logger.warning(
                 "Non-standard path(s), might not work with crontab installed "
                 "by your operating system package manager")
 
-        lineage = storage.RenewableCert.new_lineage(
-            domains[0], OpenSSL.crypto.dump_certificate(
-                OpenSSL.crypto.FILETYPE_PEM, certr.body),
-            key.pem, crypto_util.dump_pyopenssl_chain(chain),
-            params, config, cli_config)
-        return lineage
+        if self.config.dry_run:
+            logger.info("Dry run: Skipping creating new lineage for %s",
+                        domains[0])
+            return None
+        else:
+            return storage.RenewableCert.new_lineage(
+                domains[0], OpenSSL.crypto.dump_certificate(
+                    OpenSSL.crypto.FILETYPE_PEM, certr.body.wrapped),
+                key.pem, crypto_util.dump_pyopenssl_chain(chain),
+                configuration.RenewerConfiguration(self.config.namespace))
 
     def save_certificate(self, certr, chain_cert,
                          cert_path, chain_path, fullchain_path):
@@ -330,7 +315,7 @@ class Client(object):
                 self.config.strict_permissions)
 
         cert_pem = OpenSSL.crypto.dump_certificate(
-            OpenSSL.crypto.FILETYPE_PEM, certr.body)
+            OpenSSL.crypto.FILETYPE_PEM, certr.body.wrapped)
         cert_file, act_cert_path = le_util.unique_file(cert_path, 0o644)
         try:
             cert_file.write(cert_pem)
@@ -407,9 +392,10 @@ class Client(object):
             logger.warning("No config is specified.")
             raise errors.Error("No config available")
 
-        redirect = config.redirect
-        hsts = config.hsts
-        uir = config.uir # Upgrade Insecure Requests
+        supported = self.installer.supported_enhancements()
+        redirect = config.redirect if "redirect" in supported else False
+        hsts = config.hsts if "ensure-http-header" in supported else False
+        uir = config.uir if "ensure-http-header" in supported else False
 
         if redirect is None:
             redirect = enhancements.ask("redirect")
@@ -550,7 +536,7 @@ def rollback(default_installer, checkpoints, config, plugins):
 
     """
     # Misconfigurations are only a slight problems... allow the user to rollback
-    installer = display_ops.pick_installer(
+    installer = plugin_selection.pick_installer(
         config, default_installer, plugins, question="Which installer "
         "should be used for rollback?")
 
@@ -562,7 +548,7 @@ def rollback(default_installer, checkpoints, config, plugins):
         installer.restart()
 
 
-def view_config_changes(config):
+def view_config_changes(config, num=None):
     """View checkpoints and associated configuration changes.
 
     .. note:: This assumes that the installation is using a Reverter object.
@@ -573,7 +559,7 @@ def view_config_changes(config):
     """
     rev = reverter.Reverter(config)
     rev.recovery_routine()
-    rev.view_config_changes()
+    rev.view_config_changes(num)
 
 
 def _save_chain(chain_pem, chain_path):
