@@ -25,6 +25,10 @@ class CertStorageError(Error):
     """Generic `.CertStorage` error."""
 
 
+class HookCommandNotFound(Error):
+    """Failed to find a hook command in the PATH."""
+
+
 # Auth Handler Errors
 class AuthorizationError(Error):
     """Authorization error."""
@@ -48,19 +52,6 @@ class FailedChallenges(AuthorizationError):
                 for achall in self.failed_achalls if achall.error is not None))
 
 
-class ContAuthError(AuthorizationError):
-    """Let's Encrypt Continuity Authenticator error."""
-
-
-class DvAuthError(AuthorizationError):
-    """Let's Encrypt DV Authenticator error."""
-
-
-# Authenticator - Challenge specific errors
-class TLSSNI01Error(DvAuthError):
-    """Let's Encrypt TLSSNI01 error."""
-
-
 # Plugin Errors
 class PluginError(Error):
     """Let's Encrypt Plugin error."""
@@ -86,10 +77,6 @@ class NotSupportedError(PluginError):
     """Let's Encrypt Plugin function not supported error."""
 
 
-class RevokerError(Error):
-    """Let's Encrypt Revoker error."""
-
-
 class StandaloneBindError(Error):
     """Standalone plugin bind error."""
 
@@ -102,3 +89,8 @@ class StandaloneBindError(Error):
 
 class ConfigurationError(Error):
     """Configuration sanity error."""
+
+# NoninteractiveDisplay iDisplay plugin error:
+
+class MissingCommandlineFlag(Error):
+    """A command line argument was missing in noninteractive usage"""
